#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set pdbDir       $env(TALOS_DIR)/pdb
set zeroName     $env(SURF_DIR)/zero.dat
set atomNames    "C CA N"
set outName      rama.dat

set xw           16.0
set yw           16.0

set f1sigma	 0.6826895
set f2sigma	 0.9544997
set f3sigma	 0.9973002
set f4sigma	 0.9999366
set f5sigma	 0.9999994

#
#---------------------------------------------------------

dynInit -warn

dynSetChainMode -ignore
dynSetSegMode   -ignore

readROI -roi 1 -ndim 2 -in $zeroName
readROI -roi 2 -ndim 2 -in $zeroName

set rInfo [getROIInfo -roi 1]
set rdata [lindex $rInfo 0]
set fdata [lindex $rInfo 1]

set sInfo [lindex $rInfo 2]
set xSize [lindex $sInfo 0]
set ySize [lindex $sInfo 1]

set n     [expr $xSize*$ySize]
set count 0 

set rInfo [getROIInfo -roi 2]
set sdata [lindex $rInfo 0]

set i 1

foreach pdbName [glob $pdbDir/*.pdb] \
   {
    puts "$i $pdbName"

    set pdbSrc [dynRead -pdb -in $pdbName -atomName $atomNames]
    set pInfo  [dynGetInfo -src $pdbSrc]

    set r1     [lindex $pInfo 0]
    set rN     [lindex $pInfo 1]

    for {set res $r1} {$res < $rN} {incr res} \
       {
        set aInfo [dynGetPhiPsi -src $pdbSrc -res $res]
        set phi   [lindex $aInfo 0]
        set psi   [lindex $aInfo 1]

        if {$phi > 999.0} continue
        if {$psi > 999.0} continue

        while {$phi < -180.0} {set phi [expr $phi + 360.0]}
        while {$phi >  180.0} {set phi [expr $phi - 360.0]}

        while {$psi < -180.0} {set psi [expr $psi + 360.0]}
        while {$psi >  180.0} {set psi [expr $psi - 360.0]}

        vSurface -sPtr $rdata -val 1.0 \
          -xSize 361 -ySize 361 -xfw $xw -yfw $yw -x0 $phi -y0 $psi

        incr count
       }

    dynFree $pdbSrc
    incr i
   }

vsFix $rdata 1.0e-6 $n
vSort $sdata $rdata $n

set sum   [vSum $sdata $n]

set limit [expr (1.0 - $f2sigma)*$sum]
set rLoc  [vsIntegLoc $sdata $limit $n]
set n1    [expr int($rLoc)]

set limit [expr (1.0 - $f3sigma)*$sum]
set rLoc  [vsIntegLoc $sdata $limit $n]
set n2    [expr int($rLoc)]

if {$n1 > 0} \
   {
    set thresh [getPtrVal $sdata $n1] 
    set rms    [expr $thresh - [getPtrVal $sdata $n2]]
   } \
else \
   {
    set thresh [expr (1.0 - $f2sigma)*$count]
    set rms    [expr 0.1*$thresh] 
   }

puts "Thresh: $thresh RMS: $rms Loc: $n1 Count: $count Sum: $sum"

setParm $fdata FDF2LB  $xw     NULL_DIM
setParm $fdata FDF1LB  $yw     NULL_DIM
setParm $fdata FDTAU   $thresh NULL_DIM
setParm $fdata FDNOISE $rms    NULL_DIM

setParm $fdata FDUSER1 -180.0  NULL_DIM
setParm $fdata FDUSER2  180.0  NULL_DIM
setParm $fdata FDUSER3 -180.0  NULL_DIM
setParm $fdata FDUSER4  180.0  NULL_DIM
setParm $fdata FDUSER5  $count NULL_DIM

writeROI -roi 1 -out $outName -verb

exit 0
