#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set csDir    $env(TALOS_DIR)/pred
set classDir $env(TALOS_DIR)/class
set pdbDir   $env(TALOS_DIR)/pdb

proc initColorInfo {} \
{
   global cInfo

   set cInfo(000) white

   set cInfo(100) red
   set cInfo(110) pink
   set cInfo(111) magenta 
   set cInfo(101) firebrick 

   set cInfo(010) cyan 
   set cInfo(001) blue 
   set cInfo(011) green 
}

proc getPDBInfo { pdbTab } \
{
    global r1 rN chi phi psi q exposed minRes maxRes
    
    catch "unset phi"
    catch "unset psi"
    catch "unset chi"
    catch "unset q"
    catch "unset exposed"

   set rr1 [expr [getMin "$r1 $minRes"] - 1]
   set rrN [expr [getMax "$rN $maxRes"] + 1]

    for {set resID $rr1} {$resID <= $rrN} {incr resID} \
       {
        set phiPsi [getPhiPsiAngles $resID $pdbTab]

        set chi($resID) [lindex [getChiAngle $resID $pdbTab] 0]

        set phi($resID) [lindex $phiPsi 0]
        set psi($resID) [lindex $phiPsi 1]

        set exposed($resID) 0
       }

    set eList [gdbGetInfo $pdbTab -parms entries]

    foreach e $eList \
       {
        set resID    [gdbGet $e RESID]
        set thisAtom [gdbGet $e ATOMNAME]
        set thisQ    [gdbGet $e OCCUPANCY]
        set thisE    [gdbGet $e TEMPFACTOR]

        if {![info exists exposed($resID)]} {set exposed($resID) 0}

        switch -exact [string toupper $thisAtom] \
           {
            HN -
             N {set atomList "HN N"}
             C -
             O {set atomList "C O"}
            HA  -
            HA1 -
            HA2     {set atomList "HA HA1 HA2"}
            default {set atomList $thisAtom}
           }
 
        if {$thisE > 0.0} \
           {
            foreach atomName $atomList {set exposed($resID,$atomName) 1}
            incr exposed($resID)
            
            set thisQ [expr 0.666*$thisQ]
           } \
        else \
           {
            foreach atomName $atomList \
               {
                if {![info exists exposed($resID,$atomName)]} \
                   {
                    set exposed($resID,$atomName) 0
                   }
               }
           }

        foreach atomName $atomList {set q($resID,$atomName) $thisQ}
       }
}

proc getClassInfo { classTab } \
{
   global classInfo hInfo r1 rN minRes maxRes

   catch "unset classInfo"
   catch "unset hInfo"

   set rr1 [expr [getMin "$r1 $minRes"] - 1]
   set rrN [expr [getMax "$rN $maxRes"] + 1]

   set eList [gdbGetInfo $classTab -parms entries]

   for {set resID $rr1} {$resID <= $rrN} {incr resID} \
      {
       set classInfo($resID) None

       set hInfo($resID,HN) 0
       set hInfo($resID,CO) 0
       set hInfo($resID,SC) 0
      }

   foreach e $eList \
      {
       set resID [gdbGet $e RESID]
       set hbond [gdbGet $e HBOND]

       set classInfo($resID) [gdbGet $e CLASS]

       if {[string first "bb->" $hbond] >= 0} {set hInfo($resID,HN) 1} 
       if {[string first "bb<-" $hbond] >= 0} {set hInfo($resID,CO) 1} 
       if {[string first "sc<-" $hbond] >= 0} {set hInfo($resID,SC) 1} 
       if {[string first "sc->" $hbond] >= 0} {set hInfo($resID,SC) 1} 
      }
}

proc getCSColor { resID atomName } \
{
   global hInfo exposed cInfo

   set color white

   set loc "" 

   if {$hInfo($resID,HN)} {append loc 1} else {append loc 0}
   if {$hInfo($resID,CO)} {append loc 1} else {append loc 0}
   if {$exposed($resID,$atomName)} {append loc 1} else {append loc 0}
 
   return $cInfo($loc)
}

proc getCSInfo { e } \
{
   global obsShift predShift rcShift ssObs ssPred hmShift dShift sigma
   global resID resName atomName prevRes nextRes

   set resID     [gdbGet $e RESID]
   set resName   [gdbGet $e RESNAME]
   set atomName  [gdbGet $e ATOMNAME]
   set obsShift  [gdbGet $e CS_OBS]
   set predShift [gdbGet $e SHIFT]
   set hmShift   [gdbGet $e HM_SHIFT]
   set rcShift   [gdbGet $e RC_SHIFT]
   set dShift    [gdbGet $e CS_DIFF]
   set sigma     [gdbGet $e SIGMA]

   set ssObs     9999.0
   set ssPred    9999.0

   if {$rcShift < 999.0} \
      {
       if {$predShift < 999.0} {set ssPred [expr $predShift - $rcShift]} 
       if {$obsShift  < 999.0} {set ssObs  [expr $obsShift  - $rcShift]}
      }

   set prevRes [expr $resID - 1]
   set nextRes [expr $resID + 1]
}

set id 0

initColorInfo

puts -nonewline "VARS   INDEX RESID RESNAME ATOMNAME"
puts -nonewline " CS_OBS SHIFT RC_SHIFT HM_SHIFT CS_DIFF SS_OBS SS_PRED SIGMA"
puts -nonewline " PHI PSI PSI_PREV PHI_NEXT CHI"
puts -nonewline " Q EXPOSED DONOR ACCEPTOR CLASS COLOR SOURCE"
puts ""

puts -nonewline "FORMAT %5d %4d %-4s %-4s"
puts -nonewline " %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f"
puts -nonewline " %7.2f %7.2f %7.2f %7.2f %7.2f"
puts -nonewline " %6.2f %d %d %d %-14s %-8s %s"
puts ""
puts ""
 
set r1 First
set rN Last

set nn 0
 
foreach csName [glob $csDir/*.tab] \
   {
    set rootName  [file root [file tail $csName]]
    set className $classDir/$rootName.tab
    set pdbName   $pdbDir/$rootName.pdb

    incr nn

    puts stderr "$nn $rootName"

    set csTab    [gdbRead table -in $csName]
    set classTab [gdbRead table -in $className]
    set pdbTab   [gdbRead table -in $pdbName -pdb] 

    adjGLYHA $csTab CS_OBS

    set minRes   [getTabMin $csTab RESID]
    set maxRes   [getTabMax $csTab RESID]

    set seqInfo  [getSeqList $csTab r1 rN]
    set eList    [gdbGetInfo $csTab -parms entries]

    getPDBInfo   $pdbTab
    getClassInfo $classTab

    foreach e $eList \
       {
        getCSInfo $e

        if {![info exists idInfo($resID,$rootName)]} \
           {
            incr id
            set idInfo($resID,$rootName) $id
           }

        if {![info exists q($resID,$atomName)]} \
           {
            set q($resID,$atomName) 0.0
           }

        if {![info exists exposed($resID,$atomName)]} \
           {
            set exposed($resID,$atomName) 0
           }

        set color [getCSColor $resID $atomName]

        puts -nonewline \
         [format " %5d %4d %-4s %-4s" \
                 $idInfo($resID,$rootName) $resID $resName $atomName]

        puts -nonewline \
         [format " %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f" \
          $obsShift $predShift $rcShift $hmShift $dShift $ssObs $ssPred $sigma]

        puts -nonewline \
         [format " %7.2f %7.2f %7.2f %7.2f %7.2f" \
          $phi($resID) $psi($resID) $psi($prevRes) $phi($nextRes) $chi($resID)]

        puts -nonewline \
         [format " %6.2f %d" $q($resID,$atomName) $exposed($resID,$atomName)]

        puts -nonewline \
         [format " %.0f %.0f %-14s %-12s" \
          $hInfo($resID,HN) $hInfo($resID,CO) $classInfo($resID) $color]

        puts " $rootName"
       }

    gdbFree $csTab $classTab $pdbTab
   }

exit 0
