#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
# Crude calculation of local electric field, result in "OCCUPANCY" column.

#
#-------------------------------------------------------------------------

if {![file exists q]} {file mkdir q}

set qName  $env(SURF_DIR)/charge.tab
set qTab   [gdbRead table -in $qName]
set rad    10.0

dynInit

initResNames

set eList [gdbGetInfo $qTab -parms entries]

foreach e $eList \
   {
    set atomName [gdbGet $e ATOMNAME]
    set resName  [gdbGet $e RESNAME]
    set qI       [gdbGet $e CHARGE]

    set qInfo($resName,$atomName) $qI

    set qInfo($resName,HT1) 0.333
    set qInfo($resName,HT2) 0.333
    set qInfo($resName,HT3) 0.333

    set qInfo($resName,OT1) -0.50
    set qInfo($resName,OT2) -0.50
   }

set n 0

foreach pdbName [glob pdb/*.pdb] \
   {
    set root    [file tail $pdbName]
    set outName q/$root
    set prevRes -99999

    incr n 

    set pdbSrc  [dynRead -pdb -in $pdbName]
    set pdbInfo [dynGetInfo -src $pdbSrc]

    set minRes    [lindex $pdbInfo 0]
    set maxRes    [lindex $pdbInfo 1]
    set atomCount [lindex $pdbInfo 2]
    set resCount  [lindex $pdbInfo 3]

    for {set i 0} {$i < $atomCount} {incr i} \
       {
        set iInfo    [dynGetCoord -atomEntry $i -src $pdbSrc -verb 2]

        set resID    [lindex $iInfo 4]
        set resName  [lindex $iInfo 5]
        set atomName [lindex $iInfo 6]
                
        if {![info exists qInfo($resName,$atomName)]} \
           {
            puts stderr "No Charge Info For $root $resID $resName $atomName"
            set qInfo($resName,$atomName) 0.0
           }

        if {$qInfo($resName,$atomName) < 0.0} \
           {
            set qStr -1
           } \
        elseif {$qInfo($resName,$atomName) > 0.0} \
           {
            set qStr  1
           } \
        else \
           {
            set qStr  0
           }

#       dynSetCoord -atomEntry $i -src $pdbSrc -chargeStr $qStr

        dynSetCoord \
         -atomEntry $i -src $pdbSrc -charge $qInfo($resName,$atomName)

        set q($i) 0.0
       }

    for {set i 0} {$i < $atomCount} {incr i} \
       {
        set iInfo     [dynGetCoord -atomEntry $i -src $pdbSrc -verb 2]

        set resIDI    [lindex $iInfo 4]
        set resNameI  [lindex $iInfo 5]
        set atomNameI [lindex $iInfo 6]

        set xI        [lindex $iInfo 0]
        set yI        [lindex $iInfo 1]
        set zI        [lindex $iInfo 2]

        set qI        [lindex $iInfo 11]

        if {$prevRes != $resIDI} \
           {
            puts "$n. $root $resIDI of $maxRes"
            set prevRes $resIDI
           }

        set jList [dynSelect -src $pdbSrc -list -near $rad -x $xI -y $yI -z $zI]

        foreach j $jList \
           {
            if {$j <= $i} continue

            set jInfo [dynGetCoord -atomEntry $j -src $pdbSrc -verb 2]

            set resIDJ    [lindex $jInfo 4]
            set resNameJ  [lindex $jInfo 5]
            set atomNameJ [lindex $jInfo 6]
            set qJ        [lindex $jInfo 11]

            if {$resIDI == $resIDJ} continue

            set d [dynGetDist -src $pdbSrc -atomI $i -atomJ $j]

            if {$d == 0.0} \
               {
                puts stderr "Zero Distance:"
                puts stderr " $resIDI $resNameI $atomNameI $qI"
                puts stderr " $resIDJ $resNameJ $atomNameJ $qJ"
                continue
               }

            set q($i) [expr $q($i) + $qJ/($d*$d)]
            set q($j) [expr $q($j) + $qI/($d*$d)]
           }
       }
  
    set qList ""
 
    for {set i 0} {$i < $atomCount} {incr i} \
       {
        dynSetCoord -atomEntry $i -src $pdbSrc -occ [expr 10.0*$q($i)]
        lappend qList $q($i)
       }

    dynWrite -src $pdbSrc -out $outName -pdb
    dynFree  -src $pdbSrc

    puts "$root [getMin $qList] [getMax $qList]"
   }

exit
