#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

proc getThisResName { thisResID seqList r1 rN } \
{
    if {$thisResID < $r1 - 1 || $thisResID > $rN + 1} {return xxx}  

    if {$thisResID == $r1 - 1} {return NH3} 
    if {$thisResID == $rN + 1} {return COO} 

    return [getResName3 [lindex $seqList [expr $thisResID - $r1]]]
}

proc showCS { prefix postFix csInfoPtr } \
{
   upvar $csInfoPtr csInfo

   global resList atomList limitList limitFlag

   if {!$limitFlag} {puts ""}

   foreach resName $resList \
      {
       if {!$limitFlag} \
          {
           puts -nonewline [format %s%s%s $prefix $resName $postFix]
          }

       set i 0

       foreach atom $atomList \
          {
           set n [llength $csInfo($resName,$atom)]
           set a [getAvg $csInfo($resName,$atom)]
           set d [getStdDev $csInfo($resName,$atom)]

           if {$limitFlag} \
              {
               set b [lindex $limitList $i]

               set offset 0

               if {![strcmp $prefix xx]}  {set offset 2}
               if {![strcmp $prefix x]}   {set offset 1}
               if {![strcmp $postFix xx]} {set offset -2}
               if {![strcmp $postFix x]}  {set offset -1}

               if {abs($a) > $b} \
                  {
                   set fmt " %s%s%s %-4s % 6.2f %5.2f %4d"

                   puts stderr \
                    [format $fmt $prefix $resName $postFix $atom $a $d $n]

                   puts " $resName $atom [format %.3f $a] ADJ $offset"

                   puts \
                    " [getResName1 $resName] $atom [format %.3f $a] ADJ $offset"
                  }
              } \
           else \
              {
               puts -nonewline \
                [format " %-4s % 6.3f %5.3f %4d" $atom $a $d $n]
              }

           incr i
          }

       if {!$limitFlag} {puts ""}
      }
}

dynInit
initResNames

set limitFlag 0

set talosDir  $env(TALOS_DIR)
set csList    [glob $talosDir/shifts/*.tab]

if {![file exists pred]} {file mkdir pred} 
if {![file exists hms]}  {file mkdir hms} 

foreach csName $csList \
   {
    set rootName [file root [file tail $csName]]
    set pdbName  $talosDir/pdb/$rootName.pdb
    set outName  pred/$rootName.tab
    set hmsName  hms/$rootName.tab

    catch "exec pdb2hms.tcl -pdb $pdbName -seg None > $hmsName"

    set csTab    [gdbRead table -in $csName]
    
    adjGLYHA $csTab SHIFT

    gdbWrite $csTab cs.tmp.tab
    gdbFree  $csTab

    set pdbSrc   [dynRead -pdb -in $pdbName]
    set csSrc    [dynRead -cs  -in cs.tmp.tab]
    set hmsSrc   [dynRead -hms -in $hmsName]

    catch "file delete cs.tmp.tab"

    set pInfo  [dynGetInfo -src $pdbSrc]
    set minRes [lindex $pInfo 0]
    set maxRes [lindex $pInfo 1]

    set dInfo \
     [dynSimulate -cs $csSrc -src $pdbSrc \
                  -s1 $minRes -sN $maxRes -a1 $minRes \
                  -hms $hmsSrc -hmsNone -verb]

    set csRMS [lindex $dInfo 0]
    set csChi [lindex $dInfo 1]
    set csN   [lindex $dInfo 2]
    set csNN  [lindex $dInfo 3]

    set fmt   "%20s CS RMS: %8.2f  Chi: %8.2f N: %4d NN: %4d"

    puts [format $fmt $rootName $csRMS [expr $csChi/double($csN)] $csN $csNN]

    dynWrite -cs $csSrc -out $outName -verb
    dynFree -cs $csSrc
    dynFree -src $pdbSrc
   }

foreach csName $csList \
   {
    set rootName [file root [file tail $csName]]
    set pdbName  $talosDir/pdb/$rootName.pdb
    set inName   pred/$rootName.tab

    set csTab    [gdbRead table -in $inName]
    set eList    [gdbGetInfo $csTab -parms entries]

    set seqList  [getSeqList $csTab r1 rN]

    foreach e $eList \
       {
        set resID   [gdbGet $e RESID]
        set atom    [gdbGet $e ATOMNAME]
        set d       [gdbGet $e CS_DIFF]

        if {$d > 999} continue

        set prevName2 [getThisResName [expr $resID - 2] $seqList $r1 $rN]
        set prevName  [getThisResName [expr $resID - 1] $seqList $r1 $rN]
        set resName   [getThisResName $resID $seqList $r1 $rN]
        set nextName  [getThisResName [expr $resID + 1] $seqList $r1 $rN]
        set nextName2 [getThisResName [expr $resID + 2] $seqList $r1 $rN]

        lappend prevInfo2($prevName2,$atom) $d
        lappend prevInfo($prevName,$atom)   $d
        lappend nextInfo($nextName,$atom)   $d
        lappend nextInfo2($nextName2,$atom) $d

        lappend csInfo($resName,$atom) $d
        lappend csInfo(All,$atom)      $d
       }

    gdbFree $csTab
   }

set resList {ALA  ARG  ASN  ASP  CYS  cys GLN  GLU       HIS  ILE
             LEU  LYS  MET  PHE       SER  THR  TRP  TYR  VAL}

set limitList {0.075 0.3 0.3 0.3 0.4 0.1}
set atomList  {HA  CA  CB  C   N HN} 
showCS "" "" csInfo

set resList GLY
set limitList {0.075 0.075 0.3 0.3 0.4 0.1}
set atomList  {HA2 HA3 CA  C   N HN}
showCS "" "" csInfo

set resList PRO
set limitList {0.075 0.3 0.3 0.3}
set atomList {HA CA CB C}
showCS "" "" csInfo

set resList  All
set limitList {0.075 0.3 0.3 0.3 0.4 0.1}
set atomList  {HA  CA  CB  C   N HN}
showCS "" "" csInfo

set resList {ALA  ARG  ASN  ASP  CYS  cys  GLN  GLU  GLY  HIS  ILE
             LEU  LYS  MET  PHE  PRO  SER  THR  TRP  TYR  VAL}

set limitList {0.075 0.3 0.3 0.3 0.4 0.1}
set atomList  {HA  CA  CB  C   N HN}
showCS "" x prevInfo

set resList {NH3}

set limitList {0.075 0.3 0.3 0.3 0.4}
set atomList  {HA  CA  CB  C   N}
showCS "" x prevInfo

set resList {ALA  ARG  ASN  ASP  CYS  cys  GLN  GLU  GLY  HIS  ILE
             LEU  LYS  MET  PHE  PRO  SER  THR  TRP  TYR  VAL}

set limitList {0.075 0.3 0.3 0.3 0.4 0.1}
set atomList {HA CA CB C N HN}
showCS x "" nextInfo

set resList COO
set limitList {0.075 0.3 0.3 0.3 0.4}
set atomList {HA  CA  CB  C N}
showCS x "" nextInfo

set resList {ALA  ARG  ASN  ASP  CYS  cys  GLN  GLU  GLY  HIS  ILE
             LEU  LYS  MET  PHE  PRO  SER  THR  TRP  TYR  VAL}

set limitList {0.075 0.3 0.3 0.3 0.4 0.1}
set atomList  {HA  CA  CB  C   N HN}
showCS "" xx prevInfo2

set limitList {0.075 0.3 0.3 0.3 0.4 0.1}
set atomList {HA CA CB C N HN}
showCS xx "" nextInfo2

