#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# Create table of all PDB turns and structure elements (output to turnList.tab)

set auto_path "[split $env(TCLPATH) :] $auto_path"

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

proc getSeq { r1 rN } \
{
   global resInfo

   set seqList ""

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       append seqList $resInfo($res) 
      }

   return $seqList
}

dynInit -fast

initResNames

set resList {A R N D C c Q E G H I L K M F P S T W Y V ? TOTAL UNUSED}

foreach resName $resList {set countInfo($resName) 0}

set i 0

puts "VARS   INDEX PDB_NAME RES1 RESN SEQUENCE RES_PREV RES_NEXT CLASS SUBCLASS"
puts "FORMAT %5d   %12s     %4d  %4d  %5s      %s       %s       %-16s  %-10s"
puts ""

foreach fullName [glob pdb/*.pdb] \
   {
    set  pdbName [file tail $fullName]
    set  ssName  [file root [file tail $pdbName]].tab
    incr i

    if {![file exists pdb/$pdbName] || \
        ![file exists class/$ssName] || \
        ![file size class/$ssName]} \
       {
        puts stderr "Missing data for $pdbName"
        continue
       }

    puts stderr "$i. $pdbName"

    set ssTab   [gdbRead table -in class/$ssName]
    set pdbSrc  [dynRead -pdb -in pdb/$pdbName]
    set eListJ  [gdbGetInfo $ssTab -parms entries]
    set pdbInfo [dynGetInfo -src $pdbSrc]

    set minRes  [lindex $pdbInfo 0]
    set maxRes  [lindex $pdbInfo 1]

    for {set r1 [expr $minRes - 6]} {$r1 <= [expr $maxRes + 6]} {incr r1} \
       {
        set classInfo($r1)    "" 
        set subClassInfo($r1) None 
        set assCount($r1)  0
        set resInfo($r1)   ?
       }

    foreach entryJ $eListJ \
       {
        set resID [gdbGet $entryJ RESID]
        set resInfo($resID) [gdbGet $entryJ RESTYPE]
       }

    foreach entryJ $eListJ \
       {
#       set resName  [getResName1 [gdbGet $entryJ RESNAME]]
        set resName  [gdbGet $entryJ RESTYPE]
        set classStr [gdbGet $entryJ CLASS]
        set subStr   [gdbGet $entryJ HBOND_CLASS]
        set r1       [gdbGet $entryJ RESID]
        set r0       [expr $r1 - 1]
      
        incr countInfo($resName)
        incr countInfo(TOTAL)

        if {![strcmp $resName "?"]} continue

        set classInfo($r1)    [split $classStr ","]
        set subClassInfo($r1) [split $subStr   ","]

        foreach class $classInfo($r1) \
           {
            if {$r0 >= $minRes && ![strcasecmp $class NCap]} \
               {
                set classInfo($r0) NNCap
               } \
            elseif {$r0 >= $minRes && ![strcasecmp $class 3-10NCap]} \
               {
                set classInfo($r0) 3-10NNCap
               }
           }
       }

    set fmt "%5d %12s %4d %4d %5s %s %s %-12s %-10s"

    foreach entryJ $eListJ \
       {
        set r1 [gdbGet $entryJ RESID]

        foreach class $classInfo($r1) \
           {
            if {![strcasecmp $class None]} continue

            set classItem [split $classInfo($r1) "@"] 
            set classType [lindex $classItem 0]

            if {[llength $classItem] > 1} \
               {
                set offset [lindex $classItem 1]
 
                if {$offset != 1} continue 
               }
 
            if {[string first 5Turn $classType] >= 0} \
               {
                set rN [expr $r1 + 4]
               } \
            elseif {[string first NNCap $classType] >= 0} \
               {
                set rN [expr $r1 + 4]
               } \
            elseif {[string first CCap $classType] >= 0} \
               {
                set rN $r1
                set r1 [expr $rN - 3] 
               } \
            else \
               {
                set rN [expr $r1 + 3]
               } 

            set r2 [expr $r1 + 1]
            set r3 [expr $r1 + 2]
            set r4 [expr $r1 + 3]

            set thisClass $classType 
            set subClass  None

            switch $classType \
               {
                AlphaHelix -
                BetaSheet  \
                   {
                    foreach rT "$r2 $r3 $r4" \
                       {
                        if {[string first $classType $classInfo($rT)] < 0} \
                           {
                            set thisClass Other
                           }
                       } 
                   }

                3-10Helix \
                   {
                    foreach rT "$r2 $r3" \
                       {
                        if {[string first $classType $classInfo($rT)] < 0} \
                           {
                            set thisClass Other
                           }
                       }
                   }
 
                default {
                         set thisClass $classType 
                        }
               }

            if {![strcmp $thisClass AlphaHelix]} \
               {
                foreach subStr {Helix-N Helix-Mid Helix-C} \
                   {
                    if {[lsearch -exact $subClassInfo($r1) $subStr] >= 0} \
                       {
                        set subClass $subStr
                       }
                   } 
               }

            if {![strcmp $thisClass BetaSheet]} \
               {
                foreach subStr {Beta-A Beta-P Beta-B} \
                   {
                    if {[lsearch -exact $subClassInfo($r1) $subStr] >= 0} \
                       {
                        set subClass $subStr
                       }
                   }
               }

            set rBefore [expr $r1 - 1]
            set rAfter  [expr $rN + 1]
 
            set seqList [getSeq $r1 $rN] 
            set rPrev   [getSeq $rBefore $rBefore]
            set rNext   [getSeq $rAfter  $rAfter]

            if {[string first ? $seqList] >= 0} continue
            if {[string first ? $rPrev]   >= 0} continue
            if {[string first ? $rNext]   >= 0} continue

            for {set rT $r1} {$rT <= $rN} {incr rT} {incr assCount($rT)}

            puts \
             [format $fmt $i $pdbName $r1 $rN $seqList $rPrev $rNext \
                     $thisClass $subClass]
           }
       }

    for {set r1 $minRes} {$r1 <= $maxRes} {incr r1} \
       {
        if {!$assCount($r1)} {incr countInfo(UNUSED)}
       }

    unset classInfo
    unset assCount

    dynFree $pdbSrc
    gdbFree $ssTab
   }

set resList {A R N D C c Q E G H I L K M F P S T W Y V TOTAL UNUSED}

set fmt "DATA RESCOUNT %6s %6d %8.4f%%"

foreach resName $resList \
   {
    set p [expr 100.0*double($countInfo($resName))/$countInfo(TOTAL)]

    puts stderr [format $fmt $resName $countInfo($resName) $p]
   }

exit

